"""
   Rapid identification of taxonomic assignments (RITA)
   This program executes and combines the results of several BLAST algorithms and
   the Naive Bayes algorithm.  Sequences that are classified early in the pipeline
   are not executed with later algorithms, saving runtime.
   
   This work has been submitted for publication:
   MacDonald Norman J, Parks Donovan H, and Beiko Robert G.  Rapid identification
   of taxonomic assignments. Submitted 2011.
   
   If you have any questions or bug reports, please let us know at <beiko@cs.dal.ca>.
   
   This program depends on the installation of the Fragment Classification Package
   (available at http://kiwi.cs.dal.ca/Software_main)

   sequenceio.py exists primarily as it was an interface for tracking the status of each
   fragment in the web application.  It is used here as a simple method for determing
   which sequences remain to be classified.

"""
from fastaio import FastaIO


class SequenceIO(object):
    def __init__(self):
        pass


    def get_unfinished(self,job):
        fio = FastaIO()
        finished = job.get_finished()
        unfinished = set()
        for seqi, (seqid, seq) in enumerate(fio.read(job['sequence_filepath'])):
            if seqid not in finished:
                unfinished.add(seqid)
        return unfinished            
    
    def write_unfinished(self,job,output_filepath):
        fio = FastaIO()
        finished = job.get_finished()
        if len(finished) == 0:
            return job['sequence_filepath']
        fout = open(output_filepath,'w')
        for seqi, (seqid, seq) in enumerate(fio.read(job['sequence_filepath'])):
            if seqid not in finished:
                fout.write('>%s\n%s\n'%(seqid,seq))
        fout.close()
        return output_filepath
