"""
   Rapid identification of taxonomic assignments (RITA)
   This program executes and combines the results of several BLAST algorithms and
   the Naive Bayes algorithm.  Sequences that are classified early in the pipeline
   are not executed with later algorithms, saving runtime.
   
   This work has been submitted for publication:
   MacDonald Norman J, Parks Donovan H, and Beiko Robert G.  Rapid identification
   of taxonomic assignments. Submitted 2011.
   
   If you have any questions or bug reports, please let us know at <beiko@cs.dal.ca>.
   
   This program depends on the installation of the Fragment Classification Package
   (available at http://kiwi.cs.dal.ca/Software_main)

   taxonomy.py contains objects for reading taxonomy. This is much different from the 
   web version of RITA where taxonomy is stored in MySQL.  There are two subclasses of
   TaxonomyBase - TabTaxonomy and Taxonomy.  Taxonomy uses FCP's format for reading taxonomy
   while TabTaxonomy reads any tab-delimited properties file with appropriate headings (similar to PICA).
   TabTaxonomy is included for legacy reasons only, all new modules should use the same file
   as FCP.
"""

class TaxonomyBase(object):
    def get(self,key,rank,bReportMissingTaxonInfo=True):
        record = self.tax.get(key,None)
        if record == None and bReportMissingTaxonInfo == True:
            print "Warning: Missing taxonomic information for %s"%(key)
            return None
        try:
            t = record[self.header.index(rank)]
        except:
            return None
        
        return t
    
    def __str__(self):
        return str(self.tax)
    
    def get_keys(self):
        return self.tax.keys()



class TabTaxonomy(TaxonomyBase):
    def __init__(self,filepath):
        self.header = []
        self.tax = {}
        for ln,line in enumerate(open(filepath)):
            fields = line.strip().split('\t')
            if ln == 0 :
                self.header = [x.upper() for x in fields]
            else:
                self.tax[fields[0]] = fields
        
    def update(self,taxonomy):
        for key in taxonomy.tax:
            record = []
            for f in self.header:
                record.append(taxonomy.get(key,f))
            self.tax[key] = record

             
    





class Taxonomy(TaxonomyBase):
    def __init__(self,filepath):
        self.header = ['DOMAIN','PHYLUM','CLASS','ORDER','FAMILY','GENUS','SPECIES','STRAIN']
        self.tax = {}
        for line in open(filepath):
            nc, t= line.strip().split('\t')
            taxonomy = t.split(';')[:-1]

            self.tax[nc] = taxonomy
	    self.tax[taxonomy[-1].replace(' ','_').replace(':','_')] = taxonomy
            self.tax[taxonomy[-2].replace(' ','_').replace(':','_')] = taxonomy
    
    
